import inspect
from django.conf import settings
from django.urls import path
from importlib import import_module

from fast_drf.mixins.expose_api_viewset_mixin import ExposeApiViewsetMixin


class BasicRouter(object):
    @classmethod
    def get_urls(cls, **kwargs):
        _urls = []
        apps = settings.INSTALLED_APPS
        for app in apps:
            try:
                _model_tuple = inspect.getmembers(import_module("{0}.{1}".format(app, 'models')), inspect.isclass)
                for model_name, model in _model_tuple:
                    _model_api_config = model.exposed_api() if hasattr(model, 'exposed_api') else {}
                    if not _model_api_config:
                        continue
                    _model_api_config.update({'model': model})
                    expose_api_object = ExposeApiViewsetMixin(**_model_api_config)
                    _viewset_class = expose_api_object.get_runtime_viewset(**_model_api_config)
                    _urls += [path(str(_model_api_config['api_url']), _viewset_class.as_view(),
                                   name=str(_model_api_config['api_url']))]
            except ModuleNotFoundError:
                continue
        return _urls



from rest_framework.generics import ListCreateAPIView
from rest_framework import serializers


class ExposeApiViewsetMixin(object):
    permission_classes = None
    serializer_class = None
    viewset_class = None
    queryset = None
    model = None

    def __init__(self, *args, **kwargs):
        self.model = kwargs.get('model')
        self.serializer_class = kwargs.get('serializer_class', self.get_runtime_serializer(**kwargs))
        self.viewset_class = kwargs.get('viewset_class')
        self.permission_classes = kwargs.get('permission_classes', [])
        self.queryset = kwargs.get('queryset', [])

    def get_runtime_viewset(self, **kwargs):
        if self.viewset_class is None:
            return self.make_runtime_viewset(**kwargs)
        return self.viewset_class

    def make_runtime_viewset(self, **func_kwargs):
        class RunTimeViewset(ListCreateAPIView):
            if self.permission_classes:
                permission_classes = self.permission_classes
            serializer_class = self.serializer_class
            queryset = self.queryset

            def get(self, request, *args, **kwargs):
                return super(RunTimeViewset, self).get(request, *args, **kwargs)

        return RunTimeViewset

    def get_runtime_serializer(self, **kwargs):
        if self.serializer_class is None and self.model:
            return self.make_runtime_serializer(**kwargs)
        return self.serializer_class

    def make_runtime_serializer(self, **func_kwargs):
        class RuntimeModelSerializer(serializers.ModelSerializer):
            class Meta:
                model = self.model
                fields = '__all__'

        return RuntimeModelSerializer



class ExposeApiModelMixin(object):
    """
    ==== MODEL MIXIN ====

    This Mixin(Class) will help to generate automatic api from models with few configs
    """

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        """
        This method holds a bunch of API configs and return like following...
        {
            "api_url": "",  # (REQUIRED)
            "version": "",  # (NOT REQUIRED) EX: 'v1', 'v2' default 'v1'
            "queryset": "",  # (NOT REQUIRED) default all
            "api_viewset_class": "",  # (NOT REQUIRED) BaseViewset class
            "serializer_class": "",  # (NOT REQUIRED) default BaseEntitySerializer
            "permission_classes": "",  # (NOT REQUIRED) default set from settings
        }
        :param args:
        :param kwargs:
        :return: An empty Dictionary/False OR Full config dictionary.
        """
        api_configs = {}
        return api_configs



from django.contrib.auth.models import User
from django.db import models

from fast_drf.mixins.expose_api_model_mixin import ExposeApiModelMixin


class UserProfile(ExposeApiModelMixin, models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    name = models.CharField(max_length=128)
    phone = models.CharField(max_length=32)
    address = models.TextField(verbose_name='Address')
    dob = models.DateField(null=True)

    class Meta:
        app_label = 'test_app'

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        api_configs = {
            "api_url": "user-profiles",
        }
        return api_configs


class TestUser(UserProfile):
    class Meta:
        proxy = True
        app_label = 'core'

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        api_configs = {
            "api_url": "test-users",
        }
        return api_configs
